precision highp float;
varying highp vec2 textureCoordinate;

uniform sampler2D inputImageTexture;
uniform sampler2D inputImageTexture2;

uniform int enabled;

uniform float text_red;
uniform float text_green;
uniform float text_blue;
uniform float text_alpha;

uniform float background_red;
uniform float background_green;
uniform float background_blue;
uniform float background_alpha;

uniform float background_inset_top;
uniform float background_inset_right;
uniform float background_inset_bottom;
uniform float background_inset_left;

uniform float x;
uniform float y;
uniform float inset_x;
uniform float inset_y;
uniform float width;
uniform float height;
uniform float anchorX;
uniform float anchorY;

uniform float text_beginX;
uniform float text_beginY;
uniform float text_endX;
uniform float text_endY;

uniform float textImageWidth;
uniform float textImageHeight;

uniform float videoWidth;
uniform float videoHeight;

vec4 blendColor(in vec4 dstColor, in vec4 srcColor, in int colorBlendMode)
{
    vec3 vOne = vec3(1.0, 1.0, 1.0);
    vec3 vZero = vec3(0.0, 0.0, 0.0);
    vec3 resultFore = srcColor.rgb + dstColor.rgb * (1.0 - srcColor.a);
    if (colorBlendMode <= 1){ //default, since used most, put on top
        
    } else if (colorBlendMode == 4) {  //multiply
        resultFore = dstColor.rgb * srcColor.rgb;
    } else if (colorBlendMode == 2){    //screen
        resultFore = vOne - (vOne - dstColor.rgb) * (vOne - srcColor.rgb);
    } else if (colorBlendMode == 3){    //overlay
        resultFore = 2.0 * dstColor.rgb * srcColor.rgb;
        if (dstColor.r >= 0.5) {
            resultFore.r = 1.0 - 2.0 * (1.0 - dstColor.r) * (1.0 - srcColor.r);
        }
        if (dstColor.g >= 0.5) {
            resultFore.g = 1.0 - 2.0 * (1.0 - dstColor.g) * (1.0 - srcColor.g);
        }
        if (dstColor.b >= 0.5) {
            resultFore.b = 1.0 - 2.0 * (1.0 - dstColor.b) * (1.0 - srcColor.b);
        }
    } else if (colorBlendMode == 13){    //hardlight
        resultFore = 2.0 * dstColor.rgb * srcColor.rgb;
        if (srcColor.r >= 0.5) {
            resultFore.r = 1.0 - 2.0 * (1.0 - dstColor.r) * (1.0 - srcColor.r);
        }
        if (srcColor.g >= 0.5) {
            resultFore.g = 1.0 - 2.0 * (1.0 - dstColor.g) * (1.0 - srcColor.g);
        }
        if (srcColor.b >= 0.5) {
            resultFore.b = 1.0 - 2.0 * (1.0 - dstColor.b) * (1.0 - srcColor.b);
        }
    } else if (colorBlendMode == 14){    //softlight
        resultFore = 2.0 * dstColor.rgb * srcColor.rgb + dstColor.rgb * dstColor.rgb * (vOne - 2.0 * srcColor.rgb);
        if (srcColor.r >= 0.5) {
            resultFore.r = 2.0 * dstColor.r * (1.0 - srcColor.r) + (2.0 * srcColor.r - 1.0) * sqrt(dstColor.r);
        }
        if (srcColor.g >= 0.5) {
            resultFore.g = 2.0 * dstColor.g * (1.0 - srcColor.g) + (2.0 * srcColor.g - 1.0) * sqrt(dstColor.g);
        }
        if (srcColor.b >= 0.5) {
            resultFore.b = 2.0 * dstColor.b * (1.0 - srcColor.b) + (2.0 * srcColor.b - 1.0) * sqrt(dstColor.b);
        }
    } else if (colorBlendMode == 15){    //divide
        resultFore = vOne;
        if (srcColor.r > 0.0) {
            resultFore.r = dstColor.r / srcColor.r;
        }
        if (srcColor.g > 0.0) {
            resultFore.g = dstColor.g / srcColor.g;
        }
        if (srcColor.b > 0.0) {
            resultFore.b = dstColor.b / srcColor.b;
        }
        resultFore = min(vOne, resultFore);
    } else if (colorBlendMode == 9){    //add
        resultFore = dstColor.rgb + srcColor.rgb;
        resultFore = min(vOne, resultFore);
    } else if (colorBlendMode == 16){    //substract
        resultFore = dstColor.rgb - srcColor.rgb;
        resultFore = max(vZero, resultFore);
    } else if (colorBlendMode == 17){   //diff
        resultFore = abs(dstColor.rgb - srcColor.rgb);
    } else if (colorBlendMode == 18){   //darken
        resultFore = min(dstColor.rgb, srcColor.rgb);
    } else if (colorBlendMode == 8){   //lighten
        resultFore = max(dstColor.rgb, srcColor.rgb);
    }
    vec4 resultColor = vec4(resultFore, min(srcColor.a+dstColor.a, 1.0));
    return resultColor;
}

void main()
{
    vec2 pos = textureCoordinate;
    vec4 backgroundColor = texture2D(inputImageTexture, textureCoordinate);
    if (enabled == 0)
    {
        gl_FragColor = backgroundColor;
        return;
    }
    float true_width = width*textImageWidth;
    float true_height = height*textImageHeight;
    
    float left = x-anchorX*true_width + inset_x/videoWidth;
    float top = y-anchorY*true_height + inset_y/videoHeight;
    float right = left+true_width;
    float bottom = top+true_height;
    
    if (pos.x >= left - background_inset_left && pos.x <= right + background_inset_right && pos.y >= top - background_inset_top && pos.y<=bottom + background_inset_bottom){
        backgroundColor = blendColor(backgroundColor,vec4(background_red,background_green,background_blue,background_alpha),0);
    }
    if (pos.x>=left&&pos.x<=right&&pos.y>=top&&pos.y<=bottom)
    {
        vec2 text_uv = vec2( (pos.x-left)/true_width, (pos.y-top)/true_height);
        if (text_uv.x>=text_beginX&&text_uv.x<=text_endX&&text_uv.y>=text_beginY&&text_uv.y<=text_endY)
        {
            vec4 textColor = texture2D(inputImageTexture2, text_uv);
            textColor.r *= text_red*text_alpha;
            textColor.g *= text_green*text_alpha;
            textColor.b *= text_blue*text_alpha;
            textColor.a *= text_alpha;
            gl_FragColor = blendColor(backgroundColor,textColor,0);
        }
        else
        {
            gl_FragColor = backgroundColor;
        }
        
    }
    else
    {
        gl_FragColor = backgroundColor;
    }

    
}

